@@ -34,12 +34,14 @@ set_opts_knit = function(config) {
   config
 }
 
-get_base_format = function(format) {
-  if (is.character(format)) {
-    format = eval(parse(text = format))
-  }
+get_base_format = function(format, options = list()) {
+  if (is.character(format)) format = eval(parse(text = format))
   if (!is.function(format)) stop('The output format must be a function')
-  format
+  # make sure named elements in `options` have corresponding named arguments in
+  # the format function, unless the function has the ... argument
+  nms = names(formals(format))
+  if (!('...' %in% nms)) options = options[names(options) %in% c(nms, '')]
+  do.call(format, options)
 }
 
 load_config = function() {
@@ -101,7 +103,7 @@ mark_dirs = function(x) {
 }
 
 clean_empty_dir = function(dir) {
-  if (!dir_exists(dir)) return()
+  if (is.null(dir) || !dir_exists(dir)) return()
   files = list.files(dir, all.files = TRUE, recursive = TRUE)
   if (length(files) == 0) unlink(dir, recursive = TRUE)
 }
@@ -161,12 +163,25 @@ insert_code_chunk = function(x, before, after) {
 }
 
 insert_chapter_script = function(config, where = 'before') {
-  script = config[[sprintf('%s_chapter_script', where)]]
+  script = get_chapter_script(config, where)
   if (is.character(script)) {
-    c('```{r include=FALSE, cache=FALSE}', unlist(lapply(script, read_utf8)), '```')
+    c('```{r include=FALSE, cache=FALSE}', script, '```')
   }
 }
 
+get_chapter_script = function(config, where) {
+  script = config[[sprintf('%s_chapter_script', where)]]
+  unlist(lapply(script, read_utf8))
+}
+
+merge_chapter_script = function(config, where) {
+  if (!is.character(script <- get_chapter_script(config, where)) || length(script) == 0)
+    return('')
+  f = tempfile(fileext = '.R')
+  write_utf8(script, f)
+  f
+}
+
 check_special_chars = function(filename) {
   reg = rmarkdown:::.shell_chars_regex
   for (i in grep(reg, filename)) warning(
@@ -184,6 +199,11 @@ Rscript_render = function(file, ...) {
   if (Rscript(args) != 0) stop('Failed to compile ', file)
 }
 
+source_utf8 = function(file) {
+  if (file == '') return()
+  eval(xfun::parse_only(read_utf8(file)), envir = globalenv())
+}
+
 clean_meta = function(meta_file, files) {
   meta = readRDS(meta_file)
   for (i in setdiff(names(meta), files)) meta[[i]] = NULL
